@@ -1,12 +1,22 @@
 """
-Vector to QTM DGGS Grid Conversion Module
+Vector to QTM Module
+
+This module provides functionality to convert vector geometries to QTM grid cells with flexible input and output formats.
+
+Key Functions:
+    point2qtm: Convert point geometries to QTM cells
+    polyline2qtm: Convert line geometries to QTM cells
+    polygon2qtm: Convert polygon geometries to QTM cells with spatial predicates
+    geodataframe2qtm: Convert GeoDataFrame to QTM cells with topology support
+    vector2qtm: Main function for converting various input formats to QTM cells
+    vector2qtm_cli: Command-line interface for batch processing
 """
 
 import sys
 import os
 import argparse
 from tqdm import tqdm
-from shapely.geometry import Polygon, MultiPoint, MultiLineString, MultiPolygon
+from shapely.geometry import Polygon, MultiPoint
 import geopandas as gpd
 from vgrid.dggs import qtm
 from vgrid.conversion.dggscompact.qtmcompact import qtmcompact
@@ -14,12 +24,19 @@ from vgrid.conversion.dggs2geo.qtm2geo import qtm2geo
 from vgrid.utils.geometry import (
     check_predicate,
     shortest_point_distance,
-    shortest_polyline_distance,
-    shortest_polygon_distance,
     geodesic_dggs_to_geoseries,
 )
 from vgrid.stats.qtmstats import qtm_metrics
-from vgrid.utils.io import validate_qtm_resolution, process_input_data_vector, convert_to_output_format
+from vgrid.utils.io import (
+    validate_qtm_resolution,
+    process_input_data_vector,
+    convert_to_output_format,
+    DGGS_TYPES,
+)
+from vgrid.utils.constants import STRUCTURED_FORMATS, OUTPUT_FORMATS
+
+min_res = DGGS_TYPES["qtm"]["min_res"]
+max_res = DGGS_TYPES["qtm"]["max_res"]
 
 # QTM facet points
 p90_n180, p90_n90, p90_p0, p90_p90, p90_p180 = (
@@ -46,122 +63,108 @@ n90_n180, n90_n90, n90_p0, n90_p90, n90_p180 = (
 
 
 def point2qtm(
+    feature,
     resolution,
-    point,
     feature_properties=None,
     predicate=None,
     compact=False,
     topology=False,
     include_properties=True,
-    all_points=None,  # New parameter for topology preservation
 ):
     """
-    Convert a point geometry to a QTM grid cell.
+    Convert a point geometry to QTM grid cells.
 
-    Args:
-        resolution (int): QTM resolution [1..24]
-        point (shapely.geometry.Point): Point geometry to convert
-        feature_properties (dict, optional): Properties to include in output features
-        predicate (str, optional): Spatial predicate to apply (not used for points)
-        compact (bool, optional): Enable QTM compact mode (not used for points)
-        topology (bool, optional): Enable topology preserving mode - ensures disjoint points have disjoint QTM cells
-        include_properties (bool, optional): Whether to include properties in output
-        all_points: List of all points for topology preservation (required when topology=True)
+    Converts point or multipoint geometries to QTM grid cells at the specified resolution.
+    Each point is assigned to its containing QTM cell.
+
+    Parameters
+    ----------
+    feature : shapely.geometry.Point or shapely.geometry.MultiPoint
+        Point geometry to convert to QTM cells.
+    resolution : int
+        QTM resolution level [1..24].
+    feature_properties : dict, optional
+        Properties to include in output features.
+    predicate : str, optional
+        Spatial predicate to apply (not used for points).
+    compact : bool, optional
+        Enable QTM compact mode (not used for points).
+    topology : bool, optional
+        Enable topology preserving mode (handled by geodataframe2qtm).
+    include_properties : bool, optional
+        Whether to include properties in output.
+
+    Returns
+    -------
+    list of dict
+        List of dictionaries representing QTM cells containing the point(s).
+        Each dictionary contains QTM cell properties and geometry.
+
+    Examples
+    --------
+    >>> from shapely.geometry import Point
+    >>> point = Point(-122.4194, 37.7749)  # San Francisco
+    >>> cells = point2qtm(point, 10, {"name": "SF"})
+    >>> len(cells)
+    1
+
+    >>> from shapely.geometry import MultiPoint
+    >>> points = MultiPoint([(-122.4194, 37.7749), (-74.0060, 40.7128)])
+    >>> cells = point2qtm(points, 8)
+    >>> len(cells)
+    2
 
     Returns:
         list: List of GeoJSON feature dictionaries representing QTM cells containing the point
     """
-    # If topology preservation is enabled, calculate appropriate resolution
-    if topology:
-        if all_points is None:
-            raise ValueError("all_points parameter is required when topology=True")
-        
-        # Calculate the shortest distance between all points
-        shortest_distance = shortest_point_distance(all_points)
-        
-        # Find resolution where QTM cell size is smaller than shortest distance
-        # This ensures disjoint points have disjoint QTM cells
-        if shortest_distance > 0:
-            for res in range(1, 25):  # QTM resolution range is [1..24]
-                _, avg_edge_length, _ = qtm_metrics(res)
-                # Use a factor to ensure sufficient separation (triangle diameter is ~2x edge length)
-                triangle_diameter = avg_edge_length * 2
-                if triangle_diameter < shortest_distance:
-                    resolution = res
-                    break
-            else:
-                # If no resolution found, use the highest resolution
-                resolution = 24
-        else:
-            # Single point or no distance, use provided resolution
-            pass
-
     qtm_rows = []
-    latitude = point.y
-    longitude = point.x
-    qtm_id = qtm.latlon_to_qtm_id(latitude, longitude, resolution)
-    cell_polygon = qtm2geo(qtm_id)
-    if cell_polygon:
-        num_edges = 3
-        row = geodesic_dggs_to_geoseries(
-            "qtm", qtm_id, resolution, cell_polygon, num_edges
-        )
-        if include_properties and feature_properties:
-            row.update(feature_properties)
-        qtm_rows.append(row)
+    if feature.geom_type in ("Point"):
+        points = [feature]
+    elif feature.geom_type in ("MultiPoint"):
+        points = list(feature.geoms)
+    else:
+        return []
+
+    for point in points:
+        latitude = point.y
+        longitude = point.x
+        qtm_id = qtm.latlon_to_qtm_id(latitude, longitude, resolution)
+        cell_polygon = qtm2geo(qtm_id)
+        if cell_polygon:
+            num_edges = 3
+            row = geodesic_dggs_to_geoseries(
+                "qtm", qtm_id, resolution, cell_polygon, num_edges
+            )
+            if include_properties and feature_properties:
+                row.update(feature_properties)
+            qtm_rows.append(row)
     return qtm_rows
 
 
 def polyline2qtm(
-    resolution,
     feature,
+    resolution,
     feature_properties=None,
     predicate=None,
     compact=False,
     topology=False,
     include_properties=True,
-    all_polylines=None,  # New parameter for topology preservation
 ):
     """
     Convert line geometries (LineString, MultiLineString) to QTM grid cells.
 
     Args:
-        resolution (int): QTM resolution [1..24]
         feature (shapely.geometry.LineString or shapely.geometry.MultiLineString): Line geometry to convert
+        resolution (int): QTM resolution [1..24]
         feature_properties (dict, optional): Properties to include in output features
         predicate (str, optional): Spatial predicate to apply (not used for lines)
         compact (bool, optional): Enable QTM compact mode to reduce cell count
-        topology (bool, optional): Enable topology preserving mode - ensures disjoint polylines have disjoint QTM cells
+        topology (bool, optional): Enable topology preserving mode (handled by geodataframe2qtm)
         include_properties (bool, optional): Whether to include properties in output
-        all_polylines: List of all polylines for topology preservation (required when topology=True)
 
     Returns:
         list: List of GeoJSON feature dictionaries representing QTM cells intersecting the line
     """
-    # If topology preservation is enabled, calculate appropriate resolution
-    if topology:
-        if all_polylines is None:
-            raise ValueError("all_polylines parameter is required when topology=True")
-        
-        # Calculate the shortest distance between all polylines
-        shortest_distance = shortest_polyline_distance(all_polylines)
-        
-        # Find resolution where QTM cell size is smaller than shortest distance
-        # This ensures disjoint polylines have disjoint QTM cells
-        if shortest_distance > 0:
-            for res in range(1, 25):  # QTM resolution range is [1..24]
-                _, avg_edge_length, _ = qtm_metrics(res)
-                # Use a factor to ensure sufficient separation (triangle diameter is ~2x edge length)
-                triangle_diameter = avg_edge_length * 4  # in case there are 2 cells representing the same line segment
-                if triangle_diameter < shortest_distance:
-                    resolution = res
-                    break
-            else:
-                # If no resolution found, use the highest resolution
-                resolution = 24
-        else:
-            # Single polyline or no distance, use provided resolution
-            pass
     qtm_rows = []
     if feature.geom_type in ("LineString"):
         polylines = [feature]
@@ -221,55 +224,29 @@ def polyline2qtm(
 
 
 def polygon2qtm(
-    resolution,
     feature,
+    resolution,
     feature_properties=None,
     predicate=None,
     compact=False,
     topology=False,
     include_properties=True,
-    all_polygons=None,  # New parameter for topology preservation
 ):
     """
     Convert polygon geometries (Polygon, MultiPolygon) to QTM grid cells.
 
     Args:
-        resolution (int): QTM resolution [1..24]
         feature (shapely.geometry.Polygon or shapely.geometry.MultiPolygon): Polygon geometry to convert
+        resolution (int): QTM resolution [1..24]
         feature_properties (dict, optional): Properties to include in output features
         predicate (str, optional): Spatial predicate to apply ('intersect', 'within', 'centroid_within', 'largest_overlap')
         compact (bool, optional): Enable QTM compact mode to reduce cell count
-        topology (bool, optional): Enable topology preserving mode - ensures disjoint polygons have disjoint QTM cells
+        topology (bool, optional): Enable topology preserving mode (handled by geodataframe2qtm)
         include_properties (bool, optional): Whether to include properties in output
-        all_polygons: List of all polygons for topology preservation (required when topology=True)
 
     Returns:
         list: List of GeoJSON feature dictionaries representing QTM cells based on predicate
     """
-    # If topology preservation is enabled, calculate appropriate resolution
-    if topology:
-        if all_polygons is None:
-            raise ValueError("all_polygons parameter is required when topology=True")
-        
-        # Calculate the shortest distance between all polygons
-        shortest_distance = shortest_polygon_distance(all_polygons)
-        print(shortest_distance)
-        # Find resolution where QTM cell size is smaller than shortest distance
-        # This ensures disjoint polygons have disjoint QTM cells
-        if shortest_distance > 0:
-            for res in range(1, 25):  # QTM resolution range is [1..24]
-                _, avg_edge_length, _ = qtm_metrics(res)
-                # Use a factor to ensure sufficient separation (triangle diameter is ~2x edge length)
-                triangle_diameter = avg_edge_length * 4
-                if triangle_diameter < shortest_distance:
-                    resolution = res
-                    break
-            else:
-                # If no resolution found, use the highest resolution
-                resolution = 24
-        else:
-            # Single polygon or no distance, use provided resolution
-            pass
     qtm_rows = []
     if feature.geom_type in ("Polygon"):
         polygons = [feature]
@@ -332,181 +309,9 @@ def polygon2qtm(
     return qtm_rows
 
 
-def geometry2qtm(
-    geometries,
-    resolution,
-    properties_list=None,
-    predicate=None,
-    compact=False,
-    topology=False,
-    include_properties=True,
-):
-    """
-    Convert a list of geometries to QTM grid cells.
-
-    Args:
-        geometries (shapely.geometry.BaseGeometry or list): Single geometry or list of geometries
-        resolution (int): QTM resolution [1..24]
-        properties_list (list, optional): List of property dictionaries for each geometry
-        predicate (str, optional): Spatial predicate to apply for polygons
-        compact (bool, optional): Enable QTM compact mode for polygons and lines
-        topology (bool, optional): Enable topology preserving mode
-        include_properties (bool, optional): Whether to include properties in output
-
-    Returns:
-        dict: GeoJSON FeatureCollection with QTM grid cells
-    """
-    # Handle single geometry or list of geometries
-    if not isinstance(geometries, list):
-        geometries = [geometries]
-
-    # Handle properties
-    if properties_list is None:
-        properties_list = [{} for _ in geometries]
-    elif not isinstance(properties_list, list):
-        properties_list = [properties_list for _ in geometries]
-
-    # Collect all points, polylines, and polygons for topology preservation if needed
-    all_points = None
-    all_polylines = None
-    all_polygons = None
-    if topology:
-        points_list = []
-        polylines_list = []
-        polygons_list = []
-        for i, geom in enumerate(geometries):
-            if geom is None:
-                continue
-            if geom.geom_type == "Point":
-                points_list.append(geom)
-            elif geom.geom_type == "MultiPoint":
-                points_list.extend(list(geom.geoms))
-            elif geom.geom_type == "LineString":
-                polylines_list.append(geom)
-            elif geom.geom_type == "MultiLineString":
-                polylines_list.extend(list(geom.geoms))
-            elif geom.geom_type == "Polygon":
-                polygons_list.append(geom)
-            elif geom.geom_type == "MultiPolygon":
-                polygons_list.extend(list(geom.geoms))
-        if points_list:
-            all_points = MultiPoint(points_list)
-        if polylines_list:
-            all_polylines = MultiLineString(polylines_list)
-        if polygons_list:
-            all_polygons = MultiPolygon(polygons_list)
-
-    qtm_rows = []
-    for idx, geom in enumerate(tqdm(geometries, desc="Processing features")):
-        props = (
-            properties_list[idx]
-            if properties_list and idx < len(properties_list)
-            else {}
-        )
-        if not include_properties:
-            props = {}
-        if geom.geom_type == "Point":
-            qtm_rows.extend(
-                point2qtm(
-                    resolution,
-                    geom,
-                    props,
-                    predicate,
-                    compact,
-                    topology,
-                    include_properties,
-                    all_points,  # Pass all points for topology preservation
-                )
-            )
-        elif geom.geom_type == "MultiPoint":
-            for pt in geom.geoms:
-                qtm_rows.extend(
-                    point2qtm(
-                        resolution,
-                        pt,
-                        props,
-                        predicate,
-                        compact,
-                        topology,
-                        include_properties,
-                        all_points,  # Pass all points for topology preservation
-                    )
-                )
-        elif geom.geom_type in ("LineString", "MultiLineString"):
-            qtm_rows.extend(
-                polyline2qtm(
-                    resolution,
-                    geom,
-                    props,
-                    predicate,
-                    compact,
-                    topology,
-                    include_properties,
-                    all_polylines,  # Pass all polylines for topology preservation
-                )
-            )
-        elif geom.geom_type in ("Polygon", "MultiPolygon"):
-            qtm_rows.extend(
-                polygon2qtm(
-                    resolution,
-                    geom,
-                    props,
-                    predicate,
-                    compact,
-                    topology,
-                    include_properties,
-                    all_polygons=all_polygons,  # Pass all polygons for topology preservation
-                )
-            )
-    return gpd.GeoDataFrame(qtm_rows, geometry="geometry", crs="EPSG:4326")
-
-
-def dataframe2qtm(
-    df,
-    resolution,
-    predicate=None,
-    compact=False,
-    topology=False,
-    include_properties=True,
-):
-    """
-    Convert a pandas DataFrame with geometry column to QTM grid cells.
-
-    Args:
-        df (pandas.DataFrame): DataFrame with geometry column
-        resolution (int): QTM resolution [1..24]
-        predicate (str, optional): Spatial predicate to apply for polygons
-        compact (bool, optional): Enable QTM compact mode for polygons and lines
-        topology (bool, optional): Enable topology preserving mode
-        include_properties (bool, optional): Whether to include properties in output
-
-    Returns:
-        dict: GeoJSON FeatureCollection with QTM grid cells
-    """
-    geometries = []
-    properties_list = []
-    for idx, row in df.iterrows():
-        geom = row.geometry if "geometry" in row else row["geometry"]
-        if geom is not None:
-            geometries.append(geom)
-            props = row.to_dict()
-            if "geometry" in props:
-                del props["geometry"]
-            properties_list.append(props)
-    return geometry2qtm(
-        geometries,
-        resolution,
-        properties_list,
-        predicate,
-        compact,
-        topology,
-        include_properties,
-    )
-
-
 def geodataframe2qtm(
     gdf,
-    resolution,
+    resolution=None,
     predicate=None,
     compact=False,
     topology=False,
@@ -517,79 +322,158 @@ def geodataframe2qtm(
 
     Args:
         gdf (geopandas.GeoDataFrame): GeoDataFrame to convert
-        resolution (int): QTM resolution [1..24]
-        predicate (str, optional): Spatial predicate to apply for polygons
+        resolution (int, optional): QTM resolution [1..24]. Required when topology=False, auto-calculated when topology=True
+        predicate (str, optional): Spatial predicate to apply for polygons ('intersect', 'within', 'centroid_within', 'largest_overlap')
         compact (bool, optional): Enable QTM compact mode for polygons and lines
-        topology (bool, optional): Enable topology preserving mode
+        topology (bool, optional): Enable topology preserving mode to ensure disjoint features have disjoint QTM cells
         include_properties (bool, optional): Whether to include properties in output
 
     Returns:
-        dict: GeoJSON FeatureCollection with QTM grid cells
+        geopandas.GeoDataFrame: GeoDataFrame with QTM grid cells
     """
-    geometries = []
-    properties_list = []
-    for idx, row in gdf.iterrows():
+
+    # Process topology for points and multipoints if enabled
+    if topology:
+        resolution = None
+        # Collect all points for topology preservation
+        points_list = []
+        for _, row in gdf.iterrows():
+            geom = row.geometry
+            if geom is None:
+                continue
+            if geom.geom_type == "Point":
+                points_list.append(geom)
+            elif geom.geom_type == "MultiPoint":
+                points_list.extend(list(geom.geoms))
+
+        if points_list:
+            all_points = MultiPoint(points_list)
+
+            # Calculate the shortest distance between all points
+            shortest_distance = shortest_point_distance(all_points)
+
+            # Find resolution where QTM cell size is smaller than shortest distance
+            # This ensures disjoint points have disjoint QTM cells
+            if shortest_distance > 0:
+                for res in range(
+                    min_res, max_res + 1
+                ):  # QTM resolution range is [1..24]
+                    _, avg_edge_length, _, _ = qtm_metrics(res)
+                    # Use a factor to ensure sufficient separation (triangle diameter is ~2x edge length)
+                    triangle_diameter = avg_edge_length * 2
+                    if triangle_diameter < shortest_distance:
+                        estimated_resolution = res
+                        break
+
+        resolution = estimated_resolution
+
+    resolution = validate_qtm_resolution(resolution)
+
+    qtm_rows = []
+
+    for _, row in tqdm(gdf.iterrows(), desc="Processing features", total=len(gdf)):
         geom = row.geometry
-        if geom is not None:
-            geometries.append(geom)
-            props = row.to_dict()
-            if "geometry" in props:
-                del props["geometry"]
-            properties_list.append(props)
-    return geometry2qtm(
-        geometries,
-        resolution,
-        properties_list,
-        predicate,
-        compact,
-        topology,
-        include_properties,
-    )
+        if geom is None:
+            continue
+
+        props = row.to_dict()
+        if "geometry" in props:
+            del props["geometry"]
+
+        if not include_properties:
+            props = {}
+
+        if geom.geom_type == "Point" or geom.geom_type == "MultiPoint":
+            qtm_rows.extend(
+                point2qtm(
+                    feature=geom,
+                    resolution=resolution,
+                    feature_properties=props,
+                    predicate=predicate,
+                    compact=compact,
+                    topology=topology,  # Topology already processed above
+                    include_properties=include_properties,
+                )
+            )
+
+        elif geom.geom_type in ("LineString", "MultiLineString"):
+            qtm_rows.extend(
+                polyline2qtm(
+                    feature=geom,
+                    resolution=resolution,
+                    feature_properties=props,
+                    predicate=predicate,
+                    compact=compact,
+                    include_properties=include_properties,
+                )
+            )
+        elif geom.geom_type in ("Polygon", "MultiPolygon"):
+            qtm_rows.extend(
+                polygon2qtm(
+                    feature=geom,
+                    resolution=resolution,
+                    feature_properties=props,
+                    predicate=predicate,
+                    compact=compact,
+                    include_properties=include_properties,
+                )
+            )
+    return gpd.GeoDataFrame(qtm_rows, geometry="geometry", crs="EPSG:4326")
 
 
+# --- Main vector2qtm function ---
 def vector2qtm(
-    data,
-    resolution,
+    vector_data,
+    resolution=None,
     predicate=None,
     compact=False,
     topology=False,
-    output_format=None,
+    output_format="gpd",
     include_properties=True,
     **kwargs,
 ):
     """
     Convert vector data to QTM grid cells from various input formats.
-    If output_format is a file-based format (csv, geojson, shapefile, gpkg, parquet, geoparquet),
-    the output will be saved to a file in the current directory with a default name based on the input.
-    Otherwise, returns a Python object (GeoDataFrame, dict, etc.) depending on output_format.
+
+    Args:
+        vector_data (str, geopandas.GeoDataFrame, pandas.DataFrame, dict, or list): Input vector data
+        resolution (int, optional): QTM resolution [1..24]. Required when topology=False, auto-calculated when topology=True
+        predicate (str, optional): Spatial predicate to apply for polygons ('intersect', 'within', 'centroid_within', 'largest_overlap')
+        compact (bool, optional): Enable QTM compact mode for polygons and lines
+        topology (bool, optional): Enable topology preserving mode to ensure disjoint features have disjoint QTM cells
+        output_format (str, optional): Output format ('gpd', 'geojson', 'csv', 'shapefile', 'gpkg', 'parquet', 'geoparquet')
+        include_properties (bool, optional): Whether to include properties in output
+        **kwargs: Additional arguments passed to process_input_data_vector
+
+    Returns:
+        geopandas.GeoDataFrame, dict, or str: Output in the specified format. If output_format is a file-based format,
+        the output will be saved to a file in the current directory with a default name based on the input.
+        Otherwise, returns a Python object (GeoDataFrame, dict, etc.) depending on output_format.
     """
-    resolution = validate_qtm_resolution(resolution)
-    gdf = process_input_data_vector(data, **kwargs)
+    # Validate resolution parameter
+    if not topology and resolution is None:
+        raise ValueError("resolution parameter is required when topology=False")
+
+    # Only validate resolution if it's not None
+    if resolution is not None:
+        resolution = validate_qtm_resolution(resolution)
+
+    gdf = process_input_data_vector(vector_data, **kwargs)
     result = geodataframe2qtm(
         gdf, resolution, predicate, compact, topology, include_properties
     )
-    
+
     # Apply compaction if requested
     if compact:
         result = qtmcompact(result, qtm_id="qtm", output_format="gpd")
 
-    file_formats = ["csv", "geojson", "shapefile", "gpkg", "parquet", "geoparquet"]
     output_name = None
-    if output_format in file_formats:
-        ext_map = {
-            "csv": ".csv",
-            "geojson": ".geojson",
-            "shapefile": ".shp",
-            "gpkg": ".gpkg",
-            "parquet": ".parquet",
-            "geoparquet": ".parquet",
-        }
-        ext = ext_map.get(output_format, "")
-        if isinstance(data, str):
-            base = os.path.splitext(os.path.basename(data))[0]
-            output_name = f"{base}2qtm_{resolution}{ext}"
+    if output_format in OUTPUT_FORMATS:
+        if isinstance(vector_data, str):
+            base = os.path.splitext(os.path.basename(vector_data))[0]
+            output_name = f"{base}2qtm"
         else:
-            output_name = f"qtm_{resolution}{ext}"
+            output_name = "qtm"
     return convert_to_output_format(result, output_format, output_name)
 
 
@@ -605,9 +489,9 @@ def vector2qtm_cli():
         "-r",
         "--resolution",
         type=int,
-        choices=range(1, 25),
-        metavar="[1-24]",
-        help="QTM resolution [1..24] (1=coarsest, 24=finest)",
+        choices=range(min_res, max_res + 1),
+        metavar=f"[{min_res}..{max_res}]",
+        help=f"QTM resolution [{min_res}..{max_res}] (coarsest={min_res}, finest={max_res})",
     )
     parser.add_argument(
         "-p",
@@ -634,49 +518,27 @@ def vector2qtm_cli():
     parser.add_argument(
         "-f",
         "--output_format",
-        default="geojson",
-        choices=["geojson", "geojson_dict", "gpkg", "parquet", "csv", "shapefile", None],
-        help="Output format (default: geojson). Use 'geojson_dict' for Python dict, None for list of dicts.",
+        type=str,
+        default="gpd",
+        choices=OUTPUT_FORMATS,
+        help="Output format (default: gpd).",
     )
-    parser.add_argument("-o", "--output", help="Output file path (optional)")
+
     args = parser.parse_args()
     args.resolution = validate_qtm_resolution(args.resolution)
-    data = args.input
-    output_name = args.output
-    if not output_name and args.output_format in [
-        "geojson",
-        "gpkg",
-        "parquet",
-        "csv",
-        "shapefile",
-    ]:
-        extensions = {
-            "geojson": ".geojson",
-            "gpkg": ".gpkg",
-            "parquet": ".parquet",
-            "csv": ".csv",
-            "shapefile": ".shp",
-        }
-        output_name = f"vector2qtm{extensions.get(args.output_format, '')}"
     try:
         result = vector2qtm(
-            data,
-            args.resolution,
+            vector_data=args.input,
+            resolution=args.resolution,
             predicate=args.predicate,
             compact=args.compact,
             topology=args.topology,
             output_format=args.output_format,
-            output_name=output_name,
             include_properties=args.include_properties,
         )
-        if args.output_format == "geojson":
-            print(f"Output saved to {result}")
-        elif args.output_format == "geojson_dict":
-            print(f"GeoJSON dict generated with {len(result['features'])} features.")
-        elif args.output_format is None:
-            print(f"List of {len(result)} QTM cell dicts returned.")
-        elif output_name:
-            print(f"Output saved to {output_name}")
+        if args.output_format in STRUCTURED_FORMATS:
+            print(result)
+        # For file outputs, the utility prints the saved path
     except Exception as e:
         print(f"Error: {str(e)}", file=sys.stderr)
         sys.exit(1)
