@@ -525,7 +525,7 @@ class Eaggr(object):
             else:
                 proj_library_name = "libproj.so"
                 gdal_library_name = "libgdal.so"
-                dggs_library_name = "libeaggr.so"
+                dggs_library_name = "libdggs.so"
                 dll_dir = os.path.join(
                     os.path.dirname(__file__), "dlls", "linux-x86-64"
                 )
