@@ -968,30 +968,44 @@ def get_mgrs_resolution_and_cell_size(mgrs_id):
     return resolution, grid_size
 
 
+# def mgrscell(mgrs_id):
+#     resolution, grid_size = get_mgrs_resolution_and_cell_size(mgrs_id)
+#     zone, hemisphere, easting, northing = _mgrsToUtm(mgrs_id)
+
+#     min_x, min_y = easting, northing
+#     max_x, max_y = (
+#         min_x + grid_size,
+#         min_y + grid_size,
+#     )  # Correct max_x, max_y calculation
+
+#     # Determine UTM EPSG code
+#     if hemisphere == "N":
+#         epsg_code = 32600 + int(zone)
+#     else:
+#         epsg_code = 32700 + int(zone)
+
+#     utm_crs = CRS.from_epsg(epsg_code)
+#     wgs84_crs = CRS.from_epsg(4326)
+
+#     # Create transformer from UTM to WGS84
+#     transformer = Transformer.from_crs(utm_crs, wgs84_crs, always_xy=True)
+
+#     # Convert all four corners
+#     min_lon, min_lat = transformer.transform(min_x, min_y)
+#     max_lon, max_lat = transformer.transform(max_x, max_y)
+
+#     return min_lat, min_lon, max_lat, max_lon, resolution
+
 def mgrscell(mgrs_id):
     resolution, grid_size = get_mgrs_resolution_and_cell_size(mgrs_id)
     zone, hemisphere, easting, northing = _mgrsToUtm(mgrs_id)
 
     min_x, min_y = easting, northing
-    max_x, max_y = (
-        min_x + grid_size,
-        min_y + grid_size,
-    )  # Correct max_x, max_y calculation
+    max_x, max_y = (min_x + grid_size, min_y + grid_size)
 
-    # Determine UTM EPSG code
-    if hemisphere == "N":
-        epsg_code = 32600 + int(zone)
-    else:
-        epsg_code = 32700 + int(zone)
+    epsg = _epsgForUtm(zone, hemisphere)
 
-    utm_crs = CRS.from_epsg(epsg_code)
-    wgs84_crs = CRS.from_epsg(4326)
+    min_lon, min_lat = _transform(min_x, min_y, epsg, 4326, polar=False)
+    max_lon, max_lat = _transform(max_x, max_y, epsg, 4326, polar=False)
 
-    # Create transformer from UTM to WGS84
-    transformer = Transformer.from_crs(utm_crs, wgs84_crs, always_xy=True)
-
-    # Convert all four corners
-    min_lon, min_lat = transformer.transform(min_x, min_y)
-    max_lon, max_lat = transformer.transform(max_x, max_y)
-
-    return min_lat, min_lon, max_lat, max_lon, resolution
+    return min_lat, min_lon, max_lat, max_lon, resolution
\ No newline at end of file
