@@ -1,4 +1,4 @@
-# from vgrid.dggs.rhealpixdggs.dggs import WGS84_003
+# from rhealpixdggs.dggs import WGS84_003
 
 from numpy import array, base_repr  # pi is just for the doctests
 from scipy import integrate
@@ -693,16 +693,16 @@ class Cell(object):
 
             >>> c = rdggs.cell(['S', 0])
             >>> for p in c.vertices(plane=False):
-            ...     print(tuple(x.tolist() for x in my_round(p, 14)))
-            (150.0, -41.93785391016014)
-            (-180.0, -41.93785391016014)
-            (-149.99999999999997, -41.93785391016014)
-            (-179.99999999999991, -74.42400670199598)
+            ...     print(tuple(x.tolist() for x in my_round(p, 13)))
+            (150.0, -41.9378539101601)
+            (-180.0, -41.9378539101601)
+            (-150.0, -41.9378539101601)
+            (-179.9999999999999, -74.424006701996)
             >>> for p in c.vertices(plane=False, trim_dart=True):
-            ...     print(tuple(x.tolist() for x in my_round(p, 14)))
-            (150.0, -41.93785391016014)
-            (-149.99999999999997, -41.93785391016014)
-            (-179.99999999999991, -74.42400670199598)
+            ...     print(tuple(x.tolist() for x in my_round(p, 13)))
+            (150.0, -41.9378539101601)
+            (-150.0, -41.9378539101601)
+            (-179.9999999999999, -74.424006701996)
 
         """
         ul = self.ul_vertex(plane=True)
@@ -1023,8 +1023,10 @@ class Cell(object):
         suid = self.suid
         if suid[0] in CELLS0[1:5]:
             return "quad"
-        N = self.N_side
         # Cap check.
+        if suid == tuple(CELLS0[0]) or suid == tuple(CELLS0[5]):
+            return "cap"
+        N = self.N_side
         cap = True
         if N % 2 != 1:
             cap = False
