@@ -1,38 +1,51 @@
 import re
-import geopandas as gpd
-import platform
+from vgrid.utils.io import validate_dggal_type, validate_dggrid_type
+import shapely.geometry
 import shapely
 from pyproj import Geod
-from shapely.geometry import LineString, Point, Polygon,mapping
+from shapely.geometry import Point, Polygon, mapping
 from shapely.wkt import loads
 from vgrid.dggs.rhealpixdggs.utils import my_round
 from vgrid.dggs import s2
+from vgrid.dggs.eaggr.enums.shape_string_format import ShapeStringFormat
+from vgrid.dggs.eaggr.eaggr import Eaggr
+from vgrid.dggs.eaggr.enums.model import Model
+from vgrid.utils.antimeridian import fix_polygon
+from vgrid.utils.constants import DGGAL_TYPES, AUTHALIC_RADIUS, ICOSA_EDGE_M
+import platform
+import math
+
 if platform.system() == "Windows":
-    from vgrid.dggs.eaggr.enums.shape_string_format import ShapeStringFormat
     from vgrid.dggs.eaggr.eaggr import Eaggr
     from vgrid.dggs.eaggr.enums.model import Model
+
     isea4t_dggs = Eaggr(Model.ISEA4T)
-    from vgrid.dggs.eaggr.enums.shape_string_format import ShapeStringFormat
-    from vgrid.dggs.eaggr.eaggr import Eaggr
-    from vgrid.dggs.eaggr.enums.model import Model
-    from vgrid.utils.antimeridian import fix_polygon
     isea3h_dggs = Eaggr(Model.ISEA3H)
 
+
+from dggal import *
+
+app = Application(appGlobals=globals())
+pydggal_setup(app)
+
 # Initialize Geod with WGS84 ellipsoid
 geod = Geod(ellps="WGS84")
 # geod = Geod(a=6371007.181, f=0)  # sphere
 
+
 def fix_h3_antimeridian_cells(hex_boundary, threshold=-128):
     if any(lon < threshold for _, lon in hex_boundary):
         # Adjust all longitudes accordingly
         return [(lat, lon - 360 if lon > 0 else lon) for lat, lon in hex_boundary]
     return hex_boundary
 
+
 def fix_rhealpix_antimeridian_cells(boundary, threshold=-128):
     if any(lon < threshold for lon, _ in boundary):
         return [(lon - 360 if lon > 0 else lon, lat) for lon, lat in boundary]
     return boundary
 
+
 def rhealpix_cell_to_polygon(cell):
     vertices = [
         tuple(my_round(coord, 14) for coord in vertex)
@@ -63,15 +76,13 @@ def fix_isea4t_antimeridian_cells(isea4t_boundary, threshold=-100):
 
 
 def isea4t_cell_to_polygon(isea4t_cell):
-    if platform.system() == "Windows":
-        cell_to_shp = isea4t_dggs.convert_dggs_cell_outline_to_shape_string(
-            isea4t_cell, ShapeStringFormat.WKT
-        )
-        cell_to_shp_fixed = fix_isea4t_wkt(cell_to_shp)
-        cell_polygon = loads(cell_to_shp_fixed)
-        return cell_polygon
-    else:
-        raise NotImplementedError("isea4t_cell_to_polygon is only available on Windows.")
+    cell_to_shp = isea4t_dggs.convert_dggs_cell_outline_to_shape_string(
+        isea4t_cell, ShapeStringFormat.WKT
+    )
+    cell_to_shp_fixed = fix_isea4t_wkt(cell_to_shp)
+    cell_polygon = loads(cell_to_shp_fixed)
+    return cell_polygon
+
 
 def get_ease_resolution(ease_id):
     """Get the resolution level of an EASE cell ID."""
@@ -85,20 +96,17 @@ def get_ease_resolution(ease_id):
 
 
 def isea3h_cell_to_polygon(isea3h_cell):
-    if platform.system() == "Windows":
-        cell_to_shape = isea3h_dggs.convert_dggs_cell_outline_to_shape_string(
-            isea3h_cell, ShapeStringFormat.WKT
-        )
-        cell_to_shp_fixed = fix_isea4t_wkt(cell_to_shape)
-        cell_polygon = loads(cell_to_shp_fixed)
-        fixed_polygon = fix_polygon(cell_polygon)
-        return fixed_polygon
-    else:
-        raise NotImplementedError("isea3h_cell_to_polygon is only available on Windows.")
+    cell_to_shape = isea3h_dggs.convert_dggs_cell_outline_to_shape_string(
+        isea3h_cell, ShapeStringFormat.WKT
+    )
+    cell_to_shp_fixed = fix_isea4t_wkt(cell_to_shape)
+    cell_polygon = loads(cell_to_shp_fixed)
+    fixed_polygon = fix_polygon(cell_polygon)
+    return fixed_polygon
 
 
 def s2_cell_to_polygon(s2_id):
-    """ 
+    """
     Convert an S2 cell ID to a Shapely Polygon.
     """
     cell = s2.Cell(s2_id)
@@ -125,38 +133,36 @@ def fix_eaggr_wkt(eaggr_wkt):
     return f"POLYGON (({fixed_coords}))"
 
 
-def graticule_dggs_metrics(cell_polygon):    
+def graticule_dggs_metrics(cell_polygon):
     min_lon, min_lat, max_lon, max_lat = cell_polygon.bounds
-    center_lat = round((min_lat + max_lat) / 2, 7)
-    center_lon = round((min_lon + max_lon) / 2, 7)
-    cell_width = round(geod.line_length([min_lon, max_lon], [min_lat, min_lat]), 3)
-    cell_height = round(geod.line_length([min_lon, min_lon], [min_lat, max_lat]), 3)
-    cell_area = round(abs(geod.geometry_area_perimeter(cell_polygon)[0]), 3)
-    cell_perimeter = round(abs(geod.geometry_area_perimeter(cell_polygon)[1]), 3)
+    center_lat = (min_lat + max_lat) / 2
+    center_lon = (min_lon + max_lon) / 2
+    cell_width = geod.line_length([min_lon, max_lon], [min_lat, min_lat])
+    cell_height = geod.line_length([min_lon, min_lon], [min_lat, max_lat])
+    cell_area = abs(geod.geometry_area_perimeter(cell_polygon)[0])
+    cell_perimeter = abs(geod.geometry_area_perimeter(cell_polygon)[1])
     return center_lat, center_lon, cell_width, cell_height, cell_area, cell_perimeter
 
 
 def geodesic_dggs_metrics(cell_polygon, num_edges):
     cell_centroid = cell_polygon.centroid
-    center_lat = round(cell_centroid.y, 7)
-    center_lon = round(cell_centroid.x, 7)
-    cell_area = round(
-        abs(geod.geometry_area_perimeter(cell_polygon)[0]), 3
-    )
-    cell_perimeter = round(abs(geod.geometry_area_perimeter(cell_polygon)[1]), 3)       
-    avg_edge_len = round(cell_perimeter / num_edges, 3)
+    center_lat = cell_centroid.y
+    center_lon = cell_centroid.x
+    cell_area = abs(geod.geometry_area_perimeter(cell_polygon)[0])
+    cell_perimeter = abs(geod.geometry_area_perimeter(cell_polygon)[1])
+    avg_edge_len = cell_perimeter / num_edges
     return center_lat, center_lon, avg_edge_len, cell_area, cell_perimeter
 
 
-def graticule_dggs_to_feature(dggs_name, cell_id, resolution, cell_polygon):
-    center_lat, center_lon, cell_width, cell_height, cell_area, cell_perimeter = graticule_dggs_metrics(
-        cell_polygon
+def graticule_dggs_to_feature(dggs_type, cell_id, resolution, cell_polygon):
+    center_lat, center_lon, cell_width, cell_height, cell_area, cell_perimeter = (
+        graticule_dggs_metrics(cell_polygon)
     )
     feature = {
         "type": "Feature",
         "geometry": mapping(cell_polygon),
         "properties": {
-            f"{dggs_name}": str(cell_id),
+            f"{dggs_type}": str(cell_id),
             "resolution": resolution,
             "center_lat": center_lat,
             "center_lon": center_lon,
@@ -169,15 +175,15 @@ def graticule_dggs_to_feature(dggs_name, cell_id, resolution, cell_polygon):
     return feature
 
 
-def geodesic_dggs_to_feature(dggs_name, cell_id, resolution, cell_polygon, num_edges):
-    center_lat, center_lon, avg_edge_len, cell_area, cell_perimeter = geodesic_dggs_metrics(
-        cell_polygon, num_edges
+def geodesic_dggs_to_feature(dggs_type, cell_id, resolution, cell_polygon, num_edges):
+    center_lat, center_lon, avg_edge_len, cell_area, cell_perimeter = (
+        geodesic_dggs_metrics(cell_polygon, num_edges)
     )
     feature = {
         "type": "Feature",
         "geometry": mapping(cell_polygon),
         "properties": {
-            f"{dggs_name}": str(cell_id),
+            f"{dggs_type}": str(cell_id),
             "resolution": resolution,
             "center_lat": center_lat,
             "center_lon": center_lon,
@@ -189,12 +195,12 @@ def geodesic_dggs_to_feature(dggs_name, cell_id, resolution, cell_polygon, num_e
     return feature
 
 
-def graticule_dggs_to_geoseries(dggs_name, cell_id, resolution, cell_polygon):
-    center_lat, center_lon, cell_width, cell_height, cell_area, cell_perimeter = graticule_dggs_metrics(
-        cell_polygon
+def graticule_dggs_to_geoseries(dggs_type, cell_id, resolution, cell_polygon):
+    center_lat, center_lon, cell_width, cell_height, cell_area, cell_perimeter = (
+        graticule_dggs_metrics(cell_polygon)
     )
     return {
-        f"{dggs_name}": str(cell_id),
+        f"{dggs_type}": str(cell_id),
         "resolution": resolution,
         "center_lat": center_lat,
         "center_lon": center_lon,
@@ -206,12 +212,12 @@ def graticule_dggs_to_geoseries(dggs_name, cell_id, resolution, cell_polygon):
     }
 
 
-def geodesic_dggs_to_geoseries(dggs_name, cell_id, resolution, cell_polygon, num_edges):
-    center_lat, center_lon, avg_edge_len, cell_area, cell_perimeter = geodesic_dggs_metrics(
-        cell_polygon, num_edges
+def geodesic_dggs_to_geoseries(dggs_type, cell_id, resolution, cell_polygon, num_edges):
+    center_lat, center_lon, avg_edge_len, cell_area, cell_perimeter = (
+        geodesic_dggs_metrics(cell_polygon, num_edges)
     )
     return {
-        f"{dggs_name}": str(cell_id),
+        f"{dggs_type}": str(cell_id),
         "resolution": resolution,
         "center_lat": center_lat,
         "center_lon": center_lon,
@@ -221,6 +227,7 @@ def geodesic_dggs_to_geoseries(dggs_name, cell_id, resolution, cell_polygon, num
         "geometry": cell_polygon,
     }
 
+
 def shortest_point_distance(points):
     """
     Calculate distances between points in a Shapely geometry.
@@ -254,153 +261,15 @@ def shortest_point_distance(points):
         lon2, lat2 = coords[1]
 
         # Calculate the distance in meters using pyproj Geod
-        distance = geod.inv(lon1, lat1, lon2, lat2)[2]  # [2] gives the distance in meters
+        distance = geod.inv(lon1, lat1, lon2, lat2)[
+            2
+        ]  # [2] gives the distance in meters
         if distance < shortest_distance:
             shortest_distance = distance
 
     return shortest_distance if shortest_distance != float("inf") else 0
 
 
-def shortest_polyline_distance(polylines):
-    """
-    Calculate the shortest distance between polylines using GeoPandas shortest_line() method.
-    If there's only one polyline, return 0.
-    If there are multiple polylines, use shortest_line() and return the shortest distance in meters.
-
-    Args:
-        polylines: Shapely LineString or MultiLineString geometry, or GeoSeries of LineStrings
-        
-    Returns:
-        float: shortest_distance between polylines in meters
-    """
-    # Handle single LineString
-    if isinstance(polylines, LineString):
-        return 0  # Single polyline has no distance to other polylines
-    
-    # Handle MultiLineString with single line
-    if hasattr(polylines, 'geoms') and len(polylines.geoms) == 1:
-        return 0
-    
-    # Handle GeoSeries
-    if hasattr(polylines, 'iloc'):
-        # Already a GeoSeries
-        line_list = list(polylines.geometry)
-        gs = polylines
-    else:
-        # Handle MultiLineString or list
-        line_list = list(polylines.geoms) if hasattr(polylines, 'geoms') else [polylines]
-        gs = gpd.GeoSeries(line_list)
-    
-    if len(line_list) < 2:
-        return 0
-    
-    # Calculate shortest distance between all pairs of polylines using shortest_line()
-    shortest_distance = float("inf")
-    
-    for i in range(len(line_list)):
-        for j in range(i+1, len(line_list)):
-            line1 = line_list[i]
-            line2 = line_list[j]
-            
-            # Check if polylines are disjoint
-            if line1.disjoint(line2):
-                try:
-                    # Create GeoSeries for shortest_line calculation
-                    gs1 = gpd.GeoSeries([line1])
-                    gs2 = gpd.GeoSeries([line2])
-                    
-                    # Get shortest line using GeoPandas method
-                    shortest_line = gs1.shortest_line(gs2, align=False).iloc[0]
-                    
-                    if shortest_line and shortest_line.length > 0:
-                            # Get the endpoints of the shortest line
-                            coords = list(shortest_line.coords)
-                            if len(coords) >= 2:
-                                lon1, lat1 = coords[0]
-                                lon2, lat2 = coords[1]
-                                
-                                # Calculate geodesic distance in meters
-                                distance = geod.inv(lon1, lat1, lon2, lat2)[2]  # [2] gives distance in meters
-                                
-                                if distance < shortest_distance:
-                                    shortest_distance = distance
-                except Exception as e:
-                    print(f"Error calculating distance between polylines {i} and {j}: {e}")
-                    continue
-    
-    return shortest_distance if shortest_distance != float("inf") else 0 
-
-
-def shortest_polygon_distance(polygons):
-    """
-    Calculate the shortest distance between polygons using GeoPandas shortest_line() method.
-    If there's only one polygon, return 0.
-    If there are multiple polygons, use shortest_line() and return the shortest distance in meters.
-
-    Args:
-        polygons: Shapely Polygon or MultiPolygon geometry, or GeoSeries of Polygons
-        
-    Returns:
-        float: shortest_distance between polygons in meters
-    """
-    # Handle single Polygon
-    if isinstance(polygons, Polygon):
-        return 0  # Single polygon has no distance to other polygons
-    
-    # Handle MultiPolygon with single polygon
-    if hasattr(polygons, 'geoms') and len(polygons.geoms) == 1:
-        return 0
-    
-    # Handle GeoSeries
-    if hasattr(polygons, 'iloc'):
-        # Already a GeoSeries
-        polygon_list = list(polygons.geometry)
-        gs = polygons
-    else:
-        # Handle MultiPolygon or list
-        polygon_list = list(polygons.geoms) if hasattr(polygons, 'geoms') else [polygons]
-        gs = gpd.GeoSeries(polygon_list)
-    
-    if len(polygon_list) < 2:
-        return 0
-    
-    # Calculate shortest distance between all pairs of polygons using shortest_line()
-    shortest_distance = float("inf")
-    
-    for i in range(len(polygon_list)):
-        for j in range(i+1, len(polygon_list)):
-            polygon1 = polygon_list[i]
-            polygon2 = polygon_list[j]
-            
-            # Check if polygons are disjoint
-            if polygon1.disjoint(polygon2):
-                try:
-                    # Create GeoSeries for shortest_line calculation
-                    gs1 = gpd.GeoSeries([polygon1])
-                    gs2 = gpd.GeoSeries([polygon2])
-                    
-                    # Get shortest line using GeoPandas method
-                    shortest_line = gs1.shortest_line(gs2, align=False).iloc[0]
-                    
-                    if shortest_line and shortest_line.length > 0:
-                            # Get the endpoints of the shortest line
-                            coords = list(shortest_line.coords)
-                            if len(coords) >= 2:
-                                lon1, lat1 = coords[0]
-                                lon2, lat2 = coords[1]
-                                
-                                # Calculate geodesic distance in meters
-                                distance = geod.inv(lon1, lat1, lon2, lat2)[2]  # [2] gives distance in meters
-                                
-                                if distance < shortest_distance:
-                                    shortest_distance = distance
-                except Exception as e:
-                    print(f"Error calculating distance between polygons {i} and {j}: {e}")
-                    continue
-    
-    return shortest_distance if shortest_distance != float("inf") else 0
-
-
 def geodesic_distance(
     lat: float, lon: float, length_meter: float
 ) -> tuple[float, float]:
@@ -539,38 +408,224 @@ def check_crossing(lon1: float, lon2: float, validate: bool = True):
 def check_crossing_geom(geom):
     """
     Check if a geometry crosses the antimeridian (180th meridian).
-    
+
     Args:
         geom: Shapely geometry (Polygon, MultiPolygon)
-    
+
     Returns:
         bool: True if any part of the geometry crosses the antimeridian, False otherwise
     """
     crossed = False
-    
-    # Handle multi-geometries
-    if hasattr(geom, 'geoms'):
-        # MultiPolygon
-        for sub_geom in geom.geoms:
-            if check_crossing_geom(sub_geom):
-                crossed = True
-                break
-        return crossed
-    
-    # Handle single Polygon
-    if geom.geom_type == 'Polygon':
-        # Check exterior ring only
-        p_init = geom.exterior.coords[0]
-        for p in range(1, len(geom.exterior.coords)):
-            px = geom.exterior.coords[p]
-            try:
-                if check_crossing(p_init[0], px[0]):
+    try:
+        # Handle None or empty geometries
+        if geom is None or geom.is_empty:
+            return False
+
+        # Handle multi-geometries
+        if hasattr(geom, "geoms"):
+            # MultiPolygon
+            for sub_geom in geom.geoms:
+                if check_crossing_geom(sub_geom):
                     crossed = True
                     break
-            except ValueError:
-                crossed = True
-                break
-            p_init = px
-    
+            return crossed
+
+        # Handle single Polygon
+        if geom.geom_type == "Polygon":
+            # Check if exterior has coordinates
+            if geom.exterior is None or len(geom.exterior.coords) == 0:
+                return False
+
+            # Check exterior ring only
+            p_init = geom.exterior.coords[0]
+            for p in range(1, len(geom.exterior.coords)):
+                px = geom.exterior.coords[p]
+                try:
+                    if check_crossing(p_init[0], px[0]):
+                        crossed = True
+                        break
+                except ValueError:
+                    crossed = True
+                    break
+                p_init = px
+    except Exception:
+        return False
+
     return crossed
 
+
+def dggal_to_geo(dggs_type: str, zone_id: str, options: dict = {}):
+    """
+    Convert a DGGAL ZoneID to a Shapely Polygon.
+
+    Args:
+        dggs_type (str): The DGGS type e.g 'gnosis','isea4r','isea9r','isea3h','isea7h','isea7h_z7',
+            'ivea4r','ivea9r','ivea3h','ivea7h','ivea7h_z7','rtea4r','rtea9r','rtea3h','rtea7h','rtea7h_z7','healpix','rhealpix'
+        zone_id (str): The zone identifier
+        options (dict): Options for geometry generation
+            - 'crs' (str | None): "5x6" or "isea" to select alternate CRS; default WGS84
+
+    Returns:
+        shapely.geometry.Polygon: Shapely Polygon or None if invalid zone
+    """
+    # Validate DGGS type
+    dggs_type = validate_dggal_type(dggs_type)
+    # Create the appropriate DGGS instance
+    dggs_class_name = DGGAL_TYPES[dggs_type]["class_name"]
+    dggrs = globals()[dggs_class_name]()
+
+    # Get zone
+    zone = dggrs.getZoneFromTextID(zone_id)
+
+    if zone == nullZone:
+        return None
+
+    # Get CRS option
+    crsOption = options.get("crs") if options else None
+    crs = 0
+
+    if crsOption:
+        if crsOption == "5x6":
+            crs = CRS(ogc, 153456)
+        elif crsOption == "isea":
+            crs = CRS(ogc, 1534)
+
+    # Get vertices based on CRS
+    if not crs or crs == CRS(ogc, 84) or crs == CRS(epsg, 4326):
+        vertices = dggrs.getZoneRefinedWGS84Vertices(zone, 0)
+        if vertices and vertices.count:
+            count = vertices.count
+            # Create list of (lon, lat) coordinates
+            coords = [
+                (float(vertices[i].lon), float(vertices[i].lat)) for i in range(count)
+            ]
+            # Close the polygon by adding the first point at the end
+            coords.append(coords[0])
+            return shapely.geometry.Polygon(coords)
+    else:
+        vertices = dggrs.getZoneRefinedCRSVertices(zone, crs, 0)
+        if vertices and vertices.count:
+            count = vertices.count
+            # Create list of (x, y) coordinates
+            coords = [
+                (float(vertices[i].x), float(vertices[i].y)) for i in range(count)
+            ]
+            # Close the polygon by adding the first point at the end
+            coords.append(coords[0])
+            return shapely.geometry.Polygon(coords)
+
+    # Return None if no valid vertices found
+    return None
+
+
+def characteristic_length_scale(
+    cell_area: float, unit: str = "m"
+):  # cell_area is in m2
+    """
+    Compute the Characteristic Length Scale (CLS) from the cell area.
+
+    Parameters
+    ----------
+    cell_area : float
+        Cell area in m²
+
+    Returns
+    -------
+    float
+        Characteristic length scale in the m
+    """
+    unit = unit.strip().lower()
+    if unit not in {"m", "km"}:
+        raise ValueError("unit must be one of {'m','km'}")
+    # average cell area (always in m²)
+    # cell_area = 4 * math.pi * AUTHALIC_RADIUS**2 / (N_cells)
+    term = math.sqrt(cell_area / math.pi) / (2.0 * AUTHALIC_RADIUS)
+
+    # Handle case where term > 1 (can happen with very small cell areas)
+    # This indicates the cell is smaller than what can be represented on the sphere
+    if term > 1:
+        # Return a very small value as the characteristic length scale
+        # This is the maximum possible CLS (half the circumference of the Earth)
+        cls = 2.0 * math.pi * AUTHALIC_RADIUS
+    else:
+        cls = 4.0 * AUTHALIC_RADIUS * math.asin(term)
+    if unit == "km":
+        cls = cls / (10**3)
+    return cls
+
+
+def characteristic_length_scale_from_num_cells(N_cells: int, unit: str = "m"):
+    """
+    Compute the Characteristic Length Scale (CLS) from the number of cells.
+
+    Parameters
+    ----------
+    N_cells : int
+        Number of cells at the given resolution.
+
+    Returns
+    -------
+    float
+        Characteristic length scale in m
+
+    """
+    # Validate unit parameter
+    unit = unit.strip().lower()
+    if unit not in {"m", "km"}:
+        raise ValueError("unit must be one of {'m','km'}")
+
+    # average cell area (always in m²)
+    cell_area = 4 * math.pi * AUTHALIC_RADIUS**2 / (N_cells)
+    term = math.sqrt(cell_area / math.pi) / (2.0 * AUTHALIC_RADIUS)
+
+    # Handle case where term > 1 (can happen with very large numbers of cells)
+    # This indicates the cell is smaller than what can be represented on the sphere
+    if term > 1:
+        # Return a very small value as the characteristic length scale
+        # This is the maximum possible CLS (half the circumference of the Earth)
+        cls = 2.0 * math.pi * AUTHALIC_RADIUS
+    else:
+        cls = 4.0 * AUTHALIC_RADIUS * math.asin(term)
+
+    if unit == "km":
+        cls = cls / (10**3)
+    return cls
+
+
+def dggrid_intercell_distance(dggs_type: str, resolution: int, unit: str = "m"):
+    """
+    Compute the intercell distance in meters for a given DGGRID type and resolution.
+    """
+    dggs_type = validate_dggrid_type(dggs_type)
+    intercell_dist = ICOSA_EDGE_M
+    aperture = 4
+    if dggs_type in ["ISEA4T", "FULLER4T"]:
+        intercell_dist /= math.sqrt(3)
+    elif dggs_type in ["ISEA3H", "FULLER3H"]:
+        aperture = 3
+    elif dggs_type in [
+        "ISEA4T",
+        "ISEA4D",
+        "ISEA4H",
+        "FULLER4T",
+        "FULLER4D",
+        "FULLER4H",
+    ]:
+        aperture = 4
+    elif dggs_type in ["ISEA7H", "FULLER7H", "IGEO7", "PLANETRISK"]:
+        aperture = 7
+    elif dggs_type in ["PLANETRISK"]:  # Resolution pattern: _43334777777777777777777
+        if resolution == 1 or resolution == 5:
+            aperture = 4
+        elif resolution > 1 and resolution < 5:
+            aperture = 3
+        else:
+            aperture = 7
+    else:
+        return None
+
+    intercell_dist /= pow(math.sqrt(aperture), resolution)
+    if unit == "km":
+        intercell_dist = intercell_dist / (10**3)
+
+    return intercell_dist
