@@ -1,3 +1,4 @@
+import warnings
 import geopandas as gpd
 import pandas as pd
 import json
@@ -6,6 +7,21 @@ import zipfile
 import tarfile
 import requests
 from urllib.parse import urlparse
+from vgrid.utils.constants import DGGS_TYPES, DGGAL_TYPES, DGGRID_TYPES
+from dggrid4py import DGGRIDv7, tool
+from vgrid.dggs.digipin import BOUNDS
+
+# Suppress pyogrio RuntimeWarnings
+warnings.filterwarnings(
+    "ignore",
+    message="driver ESRI Shapefile does not support open option DRIVER",
+    category=RuntimeWarning,
+)
+warnings.filterwarnings(
+    "ignore",
+    message="Several features with id = .* have been found. Altering it to be unique. This warning will not be emitted anymore for this layer",
+    category=RuntimeWarning,
+)
 
 
 def process_input_data_compact(input_data, id_field=None, crs="EPSG:4326"):
@@ -65,7 +81,9 @@ def process_input_data_compact(input_data, id_field=None, crs="EPSG:4326"):
     raise ValueError("Unsupported input type")
 
 
-def process_input_data_bin(data, lat_col="lat", lon_col="lon", delimiter=None, **kwargs):
+def process_input_data_bin(
+    data, lat_col="lat", lon_col="lon", delimiter=None, **kwargs
+):
     if isinstance(data, gpd.GeoDataFrame):
         return data
     elif isinstance(data, pd.DataFrame):
@@ -79,7 +97,9 @@ def process_input_data_bin(data, lat_col="lat", lon_col="lon", delimiter=None, *
             )
             return gdf
         else:
-            raise ValueError(f"DataFrame must have either a 'geometry' column or '{lat_col}' and '{lon_col}' columns.")
+            raise ValueError(
+                f"DataFrame must have either a 'geometry' column or '{lat_col}' and '{lon_col}' columns."
+            )
     elif isinstance(data, list):
         gdf = gpd.GeoDataFrame.from_features(data)
         gdf.set_crs(epsg=4326, inplace=True)
@@ -112,7 +132,9 @@ def process_input_data_bin(data, lat_col="lat", lon_col="lon", delimiter=None, *
                 )
                 return gdf
             else:
-                raise ValueError(f"Tabular file must have columns '{lat_col}' and '{lon_col}'")
+                raise ValueError(
+                    f"Tabular file must have columns '{lat_col}' and '{lon_col}'"
+                )
         else:
             gdf = gpd.read_file(data, **kwargs)
             return process_input_data_bin(gdf, lat_col=lat_col, lon_col=lon_col)
@@ -134,6 +156,9 @@ def process_input_data_vector(input_data, crs="EPSG:4326", **kwargs):
     if isinstance(input_data, gpd.GeoDataFrame):
         if input_data.crs is None:
             input_data = input_data.set_crs(crs)
+        elif input_data.crs != crs:
+            # Reproject to target CRS if different
+            input_data = input_data.to_crs(crs)
         return input_data
     # 2. DataFrame with geometry column
     if isinstance(input_data, pd.DataFrame):
@@ -149,18 +174,31 @@ def process_input_data_vector(input_data, crs="EPSG:4326", **kwargs):
         gdf = gpd.GeoDataFrame.from_features(input_data["features"])
         if gdf.crs is None:
             gdf = gdf.set_crs(crs)
+        elif gdf.crs != crs:
+            # Reproject to target CRS if different
+            gdf = gdf.to_crs(crs)
         return gdf
     # 4. List of GeoJSON features
-    if isinstance(input_data, list) and len(input_data) > 0 and isinstance(input_data[0], dict):
+    if (
+        isinstance(input_data, list)
+        and len(input_data) > 0
+        and isinstance(input_data[0], dict)
+    ):
         gdf = gpd.GeoDataFrame.from_features(input_data)
         if gdf.crs is None:
             gdf = gdf.set_crs(crs)
+        elif gdf.crs != crs:
+            # Reproject to target CRS if different
+            gdf = gdf.to_crs(crs)
         return gdf
     # 5. File path or URL
     if isinstance(input_data, str):
         gdf = gpd.read_file(input_data, **kwargs)
         if gdf.crs is None:
             gdf = gdf.set_crs(crs)
+        elif gdf.crs != crs:
+            # Reproject to target CRS if different
+            gdf = gdf.to_crs(crs)
         return gdf
     raise ValueError(f"Unsupported input type for vector data: {type(input_data)}")
 
@@ -168,28 +206,34 @@ def process_input_data_vector(input_data, crs="EPSG:4326", **kwargs):
 def convert_to_output_format(gdf, output_format=None, output_name=None):
     """
     Utility to output a GeoDataFrame in various formats.
-    output_format: None, 'csv','geojson', 'shapefile', 'gpd', 'geojson_dict', 'gpkg', 'geoparquet'
-    output_name: if provided, write to file in the current directory
+    output_format: None, 'csv','geojson', 'shapefile'/'shp', 'gpd'/'geopandas'/'gdf'/'geodataframe',
+                   'geojson_dict'/'json_dict', 'gpkg'/'geopackage', 'geoparquet'/'parquet'
+    output_name: base name or full filename; if provided without extension, an appropriate extension is appended.
     """
     if output_format is None:
         return gdf.to_dict(orient="records")
-    elif output_format in ["gpd", "geopandas"]:
+    elif output_format in ["gpd", "geopandas", "gdf", "geodataframe"]:
         if gdf.crs is None:
             gdf.set_crs("EPSG:4326", inplace=True)
         return gdf
     elif output_format == "csv":
         if output_name:
+            # Append extension only if missing
+            if not output_name.lower().endswith(".csv"):
+                output_name = f"{output_name}.csv"
             output_name = os.path.join(os.getcwd(), output_name)
             gdf.to_csv(output_name, index=False)
             print(f"Output file saved as: {output_name}")
             return output_name
         else:
             return gdf.to_csv(index=False)
-    elif output_format == "geojson_dict":
+    elif output_format in ["geojson_dict", "json_dict"]:
         return gdf.__geo_interface__
-    elif output_format == "geojson":
+    elif output_format in ["geojson", "json"]:
         geojson = gdf.__geo_interface__
         if output_name:
+            if not output_name.lower().endswith(".geojson"):
+                output_name = f"{output_name}.geojson"
             output_name = os.path.join(os.getcwd(), output_name)
             with open(output_name, "w", encoding="utf-8") as f:
                 json.dump(geojson, f, indent=2)
@@ -197,20 +241,24 @@ def convert_to_output_format(gdf, output_format=None, output_name=None):
             return output_name
         else:
             return geojson
-    elif output_format == "shapefile":
+    elif output_format in ["shapefile", "shp"]:
         if not output_name:
             output_name = os.path.join(os.getcwd(), "output.shp")
         else:
+            if not output_name.lower().endswith(".shp"):
+                output_name = f"{output_name}.shp"
             output_name = os.path.join(os.getcwd(), output_name)
         if gdf.crs is None:
             gdf.set_crs("EPSG:4326", inplace=True)
         gdf.to_file(output_name, driver="ESRI Shapefile")
         print(f"Output file saved as: {output_name}")
         return output_name
-    elif output_format == "gpkg":
+    elif output_format in ["gpkg", "geopackage"]:
         if not output_name:
             output_name = os.path.join(os.getcwd(), "output.gpkg")
         else:
+            if not output_name.lower().endswith(".gpkg"):
+                output_name = f"{output_name}.gpkg"
             output_name = os.path.join(os.getcwd(), output_name)
         if gdf.crs is None:
             gdf.set_crs("EPSG:4326", inplace=True)
@@ -221,6 +269,8 @@ def convert_to_output_format(gdf, output_format=None, output_name=None):
         if not output_name:
             output_name = os.path.join(os.getcwd(), "output.parquet")
         else:
+            if not output_name.lower().endswith(".parquet"):
+                output_name = f"{output_name}.parquet"
             output_name = os.path.join(os.getcwd(), output_name)
         if gdf.crs is None:
             gdf.set_crs("EPSG:4326", inplace=True)
@@ -230,146 +280,257 @@ def convert_to_output_format(gdf, output_format=None, output_name=None):
     else:
         raise ValueError(f"Unsupported output_format: {output_format}")
 
+
+def dggal_convert_to_output_format(gdf, output_format=None, crs=None, output_name=None):
+    """
+    Utility to output a GeoDataFrame in various formats.
+    output_format: None, 'csv','geojson', 'shapefile'/'shp', 'gpd'/'geopandas'/'gdf'/'geodataframe',
+                   'geojson_dict'/'json_dict', 'gpkg'/'geopackage', 'geoparquet'/'parquet'
+    output_name: base name or full filename; if provided without extension, an appropriate extension is appended.
+    """
+    if crs:
+        gdf.set_crs(f"proj={crs}", inplace=True)
+    else:
+        gdf.set_crs("EPSG:4326", inplace=True)
+
+    if output_format is None:
+        return gdf.to_dict(orient="records")
+
+    elif output_format in ["gpd", "geopandas", "gdf", "geodataframe"]:
+        return gdf
+    elif output_format == "csv":
+        if output_name:
+            # Append extension only if missing
+            if not output_name.lower().endswith(".csv"):
+                output_name = f"{output_name}.csv"
+            output_name = os.path.join(os.getcwd(), output_name)
+            gdf.to_csv(output_name, index=False)
+            print(f"Output file saved as: {output_name}")
+            return output_name
+        else:
+            return gdf.to_csv(index=False)
+    elif output_format in ["geojson_dict", "json_dict"]:
+        return gdf.__geo_interface__
+    elif output_format in ["geojson", "json"]:
+        geojson = gdf.__geo_interface__
+        if output_name:
+            if not output_name.lower().endswith(".geojson"):
+                output_name = f"{output_name}.geojson"
+            output_name = os.path.join(os.getcwd(), output_name)
+            with open(output_name, "w", encoding="utf-8") as f:
+                json.dump(geojson, f, indent=2)
+            print(f"Output file saved as: {output_name}")
+            return output_name
+        else:
+            return geojson
+    elif output_format in ["shapefile", "shp"]:
+        if not output_name:
+            output_name = os.path.join(os.getcwd(), "output.shp")
+        else:
+            if not output_name.lower().endswith(".shp"):
+                output_name = f"{output_name}.shp"
+            output_name = os.path.join(os.getcwd(), output_name)
+
+        gdf.to_file(output_name, driver="ESRI Shapefile")
+        print(f"Output file saved as: {output_name}")
+        return output_name
+    elif output_format in ["gpkg", "geopackage"]:
+        if not output_name:
+            output_name = os.path.join(os.getcwd(), "output.gpkg")
+        else:
+            if not output_name.lower().endswith(".gpkg"):
+                output_name = f"{output_name}.gpkg"
+            output_name = os.path.join(os.getcwd(), output_name)
+
+        gdf.to_file(output_name, driver="GPKG")
+        print(f"Output file saved as: {output_name}")
+        return output_name
+    elif output_format in ["geoparquet", "parquet"]:
+        if not output_name:
+            output_name = os.path.join(os.getcwd(), "output.parquet")
+        else:
+            if not output_name.lower().endswith(".parquet"):
+                output_name = f"{output_name}.parquet"
+            output_name = os.path.join(os.getcwd(), output_name)
+
+        gdf.to_parquet(output_name)
+        print(f"Output file saved as: {output_name}")
+        return output_name
+    else:
+        raise ValueError(f"Unsupported output_format: {output_format}")
+
+
+def create_dggrid_instance(executable=None, **kwargs):
+    """
+    Create a DGGRIDv7 instance with proper configuration.
+
+    Args:
+        executable: Path to DGGRID executable (optional)
+        **kwargs: Additional parameters to override defaults
+
+    Returns:
+        DGGRIDv7 instance with proper configuration
+    """
+
+    if executable is None:
+        executable = tool.get_portable_executable(".")
+
+    dggrid_instance = DGGRIDv7(
+        executable=executable,
+        working_dir=".",
+        capture_logs=True,
+        silent=True,
+        has_gdal=False,
+        tmp_geo_out_legacy=True,
+        debug=False,
+    )
+    return dggrid_instance
+
+
 # =============================
 # Resolution Validators
 # =============================
+def validate_dggs_type(dggs_type: str):
+    dggs_type = dggs_type.strip().lower()
+    available_types = list(DGGS_TYPES.keys())
+    if dggs_type not in available_types:
+        raise ValueError(
+            f"Invalid DGGS type '{dggs_type}'. Valid types are: {available_types}"
+        )
+    return dggs_type
 
-def validate_h3_resolution(resolution):
+
+def validate_dggs_resolution(dggs_type: str, resolution: int) -> int:
     """
-    Validate that H3 resolution is in the valid range [0..15].
+    Validate resolution for a DGGS type using DGGS_TYPES uniform (min, max) bounds.
+
     Args:
-        resolution: Resolution value to validate
+        dggs_type: DGGS name present in DGGS_TYPES
+        resolution: integer resolution value
+
     Returns:
-        int: Validated resolution value
+        int: Validated resolution
+
     Raises:
-        ValueError: If resolution is not in range [0..15]
-        TypeError: If resolution is not an integer
+        ValueError: If DGGS type unknown or resolution out of bounds
+        TypeError: If inputs are of incorrect types
     """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
+    dggs_type = validate_dggs_type(dggs_type)
+    min_res = int(DGGS_TYPES[dggs_type]["min_res"])
+    max_res = int(DGGS_TYPES[dggs_type]["max_res"])
+
+    if resolution is None or resolution < min_res or resolution > max_res:
+        raise ValueError(
+            f"Resolution for '{dggs_type}' must be in range [{min_res}..{max_res}], got {resolution}"
         )
-    if resolution < 0 or resolution > 15:
-        raise ValueError(f"Resolution must be in range [0..15], got {resolution}")
     return resolution
 
-def validate_a5_resolution(resolution):
+def validate_coordinate(min_lat, min_lon, max_lat, max_lon):
+    if min_lat < -90:
+        min_lat = -90
+    if min_lon < -180:
+        min_lon = -180
+    if max_lat > 90:
+        max_lat = 90
+    if max_lon > 180:
+        max_lon = 180     
+    return min_lat, min_lon, max_lat, max_lon
+
+
+def validate_digipin_coordinate(min_lat, min_lon, max_lat, max_lon):
     """
-    Validate that A5 resolution is in the valid range [0..29].
+    Validate that coordinates are within India's bounds.
+    
     Args:
-        resolution: Resolution value to validate
+        min_lat (float): Minimum latitude coordinate
+        min_lon (float): Minimum longitude coordinate
+        max_lat (float): Maximum latitude coordinate
+        max_lon (float): Maximum longitude coordinate
+        
     Returns:
-        int: Validated resolution value
+        tuple: (min_lat, min_lon, max_lat, max_lon) if coordinates are valid
+        
     Raises:
-        ValueError: If resolution is not in range [0..29]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 29:
-        raise ValueError(f"Resolution must be in range [0..29], got {resolution}")
-    return resolution
+        ValueError: If coordinates are outside India's bounds
+    """  
+    
+    if not isinstance(min_lat, (int, float)) or not isinstance(min_lon, (int, float)) or not isinstance(max_lat, (int, float)) or not isinstance(max_lon, (int, float)):
+        raise TypeError("Latitude and longitude must be numeric values")
+    
+    if min_lat < BOUNDS['minLat'] or max_lat > BOUNDS['maxLat']:
+        raise ValueError(f"Latitude {min_lat} is outside India's bounds [{BOUNDS['minLat']}, {BOUNDS['maxLat']}]")
+    
+    if min_lon < BOUNDS['minLon'] or max_lon > BOUNDS['maxLon']:
+        raise ValueError(f"Longitude {min_lon} is outside India's bounds [{BOUNDS['minLon']}, {BOUNDS['maxLon']}]")
+    
+    return min_lat, min_lon, max_lat, max_lon
 
 
+def validate_h3_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("h3", resolution)
 
-def validate_geohash_resolution(resolution):
-    """
-    Validate that Geohash resolution is in the valid range [1..12] (1=coarsest, 12=finest).
-    Args:
-        resolution (int): Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [1..12]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 1 or resolution > 12:
-        raise ValueError(f"Resolution must be in range [1..12], got {resolution}")
-    return resolution
 
-def validate_ease_resolution(resolution):
-    """
-    Validate that EASE resolution is in the valid range [0..6].
-    Args:
-        resolution (int): Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..6]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 6:
-        raise ValueError(f"Resolution must be in range [0..6], got {resolution}")
-    return resolution
+def validate_s2_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("s2", resolution)
 
-def validate_isea3h_resolution(resolution):
-    """
-    Validate that ISEA3H resolution is in the valid range [0..32].
-    Args:
-        resolution (int): Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..32]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 40:
-        raise ValueError(f"Resolution must be in range [0..40], got {resolution}")
-    return resolution
 
-def validate_isea4t_resolution(resolution):
-    """
-    Validate that ISEA4T resolution is in the valid range [0..39].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..39]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 39:
-        raise ValueError(f"Resolution must be in range [0..39], got {resolution}")
-    return resolution
+def validate_a5_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("a5", resolution)
 
-def validate_qtm_resolution(resolution):
-    """
-    Validate that QTM resolution is in the valid range [1..24].
-    Args:
-        resolution (int): Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [1..24]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 1 or resolution > 24:
-        raise ValueError(f"Resolution must be in range [1..24], got {resolution}")
-    return resolution
 
-def validate_olc_resolution(resolution):
+# def validate_healpix_resolution(resolution: int) -> int:
+#     return validate_dggs_resolution("healpix", resolution)
+
+
+def validate_rhealpix_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("rhealpix", resolution)
+
+
+def validate_isea3h_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("isea3h", resolution)
+
+
+def validate_isea4t_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("isea4t", resolution)
+
+
+def validate_qtm_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("qtm", resolution)
+
+
+def validate_ease_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("ease", resolution)
+
+
+def validate_geohash_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("geohash", resolution)
+
+
+def validate_georef_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("georef", resolution)
+
+
+def validate_mgrs_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("mgrs", resolution)
+
+
+def validate_tilecode_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("tilecode", resolution)
+
+
+def validate_quadkey_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("quadkey", resolution)
+
+
+def validate_gars_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("gars", resolution)
+
+
+def validate_maidenhead_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("maidenhead", resolution)
+
+
+def validate_olc_resolution(resolution: int) -> int:
     """
     Validate that OLC resolution is in the valid range [2,4,6,8,10,11,12,13,14,15].
     Args:
@@ -390,197 +551,79 @@ def validate_olc_resolution(resolution):
         )
     return resolution
 
-def validate_geohash_resolution(resolution):
-    """
-    Validate that Geohash resolution is in the valid range [0..12] (0=coarsest, 12=finest).
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..12]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 12:
-        raise ValueError(f"Resolution must be in range [0..12], got {resolution}")
-    return resolution
 
-def validate_georef_resolution(resolution):
-    """
-    Validate that Georef resolution is in the valid range [0..10] (0=coarsest, 10=finest).
-    Args:       
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..10]
-        TypeError: If resolution is not an integer  
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 10:
-        raise ValueError(f"Resolution must be in range [0..10], got {resolution}")
-    return resolution
+def validate_digipin_resolution(resolution: int) -> int:
+    return validate_dggs_resolution("digipin", resolution)
 
-def validate_mgrs_resolution(resolution):
-    """
-    Validate that MGRS resolution is in the valid range [0..5] (0=coarsest, 5=finest).
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..5]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 5:
-        raise ValueError(f"Resolution must be in range [0..5], got {resolution}")
-    return resolution
 
-def validate_tilecode_resolution(resolution):
+def validate_vgrid_resolution(resolution: int) -> int:
     """
-    Validate that Tilecode resolution is in the valid range [0..29] (0=coarsest, 29=finest).
+    Validate VGRID resolution is within valid bounds.
+
     Args:
         resolution (int): Resolution value to validate
+
     Returns:
         int: Validated resolution value
+
     Raises:
-        ValueError: If resolution is not in range [0..29]
+        ValueError: If resolution is out of bounds [-31, 31]
         TypeError: If resolution is not an integer
     """
     if not isinstance(resolution, int):
         raise TypeError(
             f"Resolution must be an integer, got {type(resolution).__name__}"
         )
-    if resolution < 0 or resolution > 29:
-        raise ValueError(f"Resolution must be in range [0..29], got {resolution}")
+    if resolution < -31:  # -MAX_HIERARCHY_LEVEL for VGRID
+        raise ValueError(f"Resolution must be >= -31, got {resolution}")
+    if resolution > 31:  # MAX_HIERARCHY_LEVEL for VGRID
+        raise ValueError(f"Resolution must be <= 31, got {resolution}")
     return resolution
 
-def validate_quadkey_resolution(resolution):
-    """
-    Validate that Quadkey resolution is in the valid range [0..29] (0=coarsest, 29=finest).
-    Args:
-        resolution (int): Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..29] (0=coarsest, 29=finest)
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 29:
+
+def validate_dggal_type(dggs_type: str):
+    dggs_type = dggs_type.strip().lower()
+    available_types = list(DGGAL_TYPES.keys())
+    if dggs_type not in available_types:
         raise ValueError(
-            f"Resolution must be in range [0..29] (0=coarsest, 29=finest), got {resolution}"
+            f"Invalid DGGAL type '{dggs_type}'. Valid types are: {available_types}"
+        )
+    return dggs_type
+
+
+def validate_dggal_resolution(dggs_type: str, resolution: int) -> int:
+    dggs_type = validate_dggal_type(dggs_type)
+
+    min_res = int(DGGAL_TYPES[dggs_type]["min_res"])
+    max_res = int(DGGAL_TYPES[dggs_type]["max_res"])
+    if resolution is None or resolution < min_res or resolution > max_res:
+        raise ValueError(
+            f"Resolution for '{dggs_type}' must be in range [{min_res}..{max_res}], got {resolution}"
         )
     return resolution
 
-def validate_maidenhead_resolution(resolution):
-    """
-    Validate that Maidenhead resolution is in the valid range [1..4].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [1..4]    
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(    
-            f"Resolution must be an integer, got {type(resolution).__name__}"
+
+def validate_dggrid_type(dggrid_type: str):
+    dggrid_type = dggrid_type.strip().upper()
+    available_types = list(DGGRID_TYPES.keys())
+    if dggrid_type not in available_types:
+        raise ValueError(
+            f"Invalid DGGRID type '{dggrid_type}'. Valid types are: {available_types}"
+        )
+    return dggrid_type
+
+
+def validate_dggrid_resolution(dggrid_type: str, resolution: int) -> int:
+    dggrid_type = validate_dggrid_type(dggrid_type)
+    min_res = int(DGGRID_TYPES[dggrid_type]["min_res"])
+    max_res = int(DGGRID_TYPES[dggrid_type]["max_res"])
+
+    if resolution is None or resolution < min_res or resolution > max_res:
+        raise ValueError(
+            f"Resolution for '{dggrid_type}' must be in range [{min_res}..{max_res}], got {resolution}"
         )
-    if resolution < 1 or resolution > 4:
-        raise ValueError("Resolution must be in range [1..4]")
     return resolution
 
-def validate_gars_resolution(resolution):
-    """
-    Validate that Maidenhead resolution is in the valid range [1..4].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [1..4]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 1 or resolution > 4:
-        raise ValueError("Resolution must be in range [1..4]")
-    return resolution
-
-def validate_healpix_resolution(resolution):
-    """
-    Validate that HEALPix resolution is in the valid range [0..29].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..29]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 29:
-        raise ValueError(f"Resolution must be in range [0..29], got {resolution}")
-    return resolution
-
-def validate_rhealpix_resolution(resolution):
-    """
-    Validate that rHEALPix resolution is in the valid range [0..15].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..15]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 15:
-        raise ValueError(f"Resolution must be in range [0..15], got {resolution}")
-    return resolution
-
-def validate_s2_resolution(resolution):
-    """
-    Validate that S2 resolution is in the valid range [0..28].
-    Args:
-        resolution: Resolution value to validate
-    Returns:
-        int: Validated resolution value
-    Raises:
-        ValueError: If resolution is not in range [0..28]
-        TypeError: If resolution is not an integer
-    """
-    if not isinstance(resolution, int):
-        raise TypeError(
-            f"Resolution must be an integer, got {type(resolution).__name__}"
-        )
-    if resolution < 0 or resolution > 28:
-        raise ValueError(f"Resolution must be in range [0..28], got {resolution}")
-    return resolution
 
 def gars_num_cells(resolution: int) -> int:
     """
@@ -599,15 +642,11 @@ def gars_num_cells(resolution: int) -> int:
     base_cells = lon_bands * lat_bands
 
     # Multipliers for each resolution
-    multipliers = {
-        1: 1,
-        2: 4,
-        3: 4 * 9,
-        4: 4 * 9 * 25
-    }
+    multipliers = {1: 1, 2: 4, 3: 4 * 9, 4: 4 * 9 * 25}
 
     return base_cells * multipliers[resolution]
 
+
 def is_url(path):
     """Check if the given path is a URL."""
     try:
@@ -616,6 +655,7 @@ def is_url(path):
     except Exception:
         return False
 
+
 def read_geojson_file(geojson_path):
     """Read GeoJSON from either a local file or URL."""
     if is_url(geojson_path):
@@ -639,6 +679,7 @@ def read_geojson_file(geojson_path):
             print(f"Error reading GeoJSON file: {e}")
             return None
 
+
 def github_raw_url(url):
     """Get the raw URL for a GitHub file.
 
@@ -653,6 +694,7 @@ def github_raw_url(url):
         )
     return url
 
+
 def install_package(package):
     """Install a Python package.
 
@@ -685,6 +727,7 @@ def install_package(package):
         # Wait for process to complete
         process.wait()
 
+
 def extract_archive(archive, outdir=None, **kwargs) -> None:
     """
     Extracts a multipart archive.
@@ -731,6 +774,7 @@ def extract_archive(archive, outdir=None, **kwargs) -> None:
         print("The unzipped files might already exist. Skipping extraction.")
         return
 
+
 def download_file(
     url=None,
     output=None,
@@ -833,6 +877,7 @@ def download_file(
 
     return os.path.abspath(output)
 
+
 def download_files(
     urls,
     out_dir=None,
@@ -918,6 +963,7 @@ def download_files(
         for file in filepaths:
             os.remove(file)
 
+
 def download_folder(
     url=None,
     id=None,
@@ -955,4 +1001,4 @@ def download_folder(
     files = gdown.download_folder(
         url, id, output, quiet, proxy, speed, use_cookies, remaining_ok
     )
-    return files
\ No newline at end of file
+    return files
